package models.dataset;

import models.DataModel;
import utilities.builders.DomainModelConverter;

import java.util.Date;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

public class Dataset implements DataModel<entities.Dataset>{
	private UUID id;
    private String label;
    private String reference;
    private String uri;
    private String description;
    private short status;
    private String properties;
    private entities.DMP dmp;
    private entities.DatasetProfile profile;
    private List<Registry> registries;
	private List<Service> services;
	private List<DataRepository> dataRepositories;
		
	public UUID getId() {
		return id;
	}
	public void setId(UUID id) {
		this.id = id;
	}
	public String getLabel() {
		return label;
	}
	public void setLabel(String label) {
		this.label = label;
	}
	public String getReference() {
		return reference;
	}
		
	public void setReference(String reference) {
		this.reference = reference;
	}
	public String getUri() {
		return uri;
	}
	public void setUri(String uri) {
		this.uri = uri;
	}
	public short getStatus() {
		return status;
	}
	public void setStatus(short status) {
		this.status = status;
	}
	public String getProperties() {
		return properties;
	}
	public void setProperties(String properties) {
		this.properties = properties;
	}
	public List<Registry> getRegistries() {
		return registries;
	}
	public void setRegistries(List<Registry> registries) {
		this.registries = registries;
	}
	public List<Service> getServices() {
		return services;
	}
	public void setServices(List<Service> services) {
		this.services = services;
	}
	public List<DataRepository> getDataRepositories() {
		return dataRepositories;
	}
	public void setDataRepositories(List<DataRepository> dataRepositories) {
		this.dataRepositories = dataRepositories;
	}	
	
	
	
	public String getDescription() {
		return description;
	}
	public void setDescription(String description) {
		this.description = description;
	}
	public entities.DMP getDmp() {
		return dmp;
	}
	public void setDmp(entities.DMP dmp) {
		this.dmp = dmp;
	}
	public entities.DatasetProfile getProfile() {
		return profile;
	}
	public void setProfile(entities.DatasetProfile profile) {
		this.profile = profile;
	}
	
	public void fromDataModel(entities.Dataset entity) throws IllegalAccessException, InstantiationException{
		this.id = entity.getId();
		this.label = entity.getLabel();
		this.properties = entity.getProperties();
		this.reference = entity.getReference();
		//this.dmp = entity.getDmp();
		this.description = entity.getDescription();
		this.profile = entity.getProfile();
		this.registries = new DomainModelConverter<entities.Registry,Registry>().fromDataModel(entity.getRegistries().stream().collect(Collectors.toList()), Registry.class);	
		this.dataRepositories = new DomainModelConverter<entities.DataRepository,DataRepository>().fromDataModel(entity.getDataRepositories().stream().collect(Collectors.toList()), DataRepository.class);
		this.services = new DomainModelConverter<entities.Service,Service>().fromDataModel(entity.getServices().stream().collect(Collectors.toList()), Service.class);		
	}
	
	public entities.Dataset toDataModel(){
		entities.Dataset entity = new entities.Dataset();
		entity.setId(this.id);
		entity.setLabel(this.label);
		entity.setReference(this.reference);
		entity.setUri(this.uri);
		entity.setProperties(this.properties);
		entity.setStatus(this.status);
		entity.setDmp(dmp);
		entity.setDescription(this.description);
		entity.setProfile(profile);
		if(!this.registries.isEmpty()){
			entity.setRegistries(new HashSet<entities.Registry>());
			for(Registry registry:this.registries){
				entity.getRegistries().add(registry.toDataModel());
			}
		}
				
		if(!this.dataRepositories.isEmpty()){
			entity.setDataRepositories(new HashSet<entities.DataRepository>());
			for(DataRepository dataRepository:this.dataRepositories){
				entity.getDataRepositories().add(dataRepository.toDataModel());
			}
		}
		
		if(!this.services.isEmpty()){
			entity.setServices(new HashSet<entities.Service>());
			for(Service service:this.services){
				entity.getServices().add(service.toDataModel());
			}
		}
		
		
		return entity;
	}
	
}
