package models.user.components.datasetprofile;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import models.components.commons.DefaultValue;
import models.components.commons.Multiplicity;
import models.components.commons.ViewStyle;
import models.user.composite.PropertiesModelBuilder;
import utilities.ModelDefinition;
import utilities.ViewStyleDefinition;
import utilities.builders.ModelBuilder;

public class Field implements Comparable,PropertiesModelBuilder, ModelDefinition<entities.xmlmodels.modeldefinition.Field>,ViewStyleDefinition<entities.xmlmodels.viewstyledefinition.Field>{
	private String id;
	private Integer ordinal;
	private String title;
	private String value;
	private String description;
	private String extendedDescription;
	private ViewStyle viewStyle;
	private String datatype;
	private boolean defaultVisibility;
	private int page;
	private DefaultValue defaultValue;
	private Multiplicity multiplicity;
	private Object data;
	private List<Field> multiplicityItems;
	
	public List<Field> getMultiplicityItems() {
		return multiplicityItems;
	}
	public void setMultiplicityItems(List<Field> multiplicityItems) {
		this.multiplicityItems = multiplicityItems;
	}
	public void setOrdinal(Integer ordinal) {
		this.ordinal = ordinal;
	}
	public String getId() {
		return id;
	}
	public void setId(String id) {
		this.id = id;
	}
	
	public int getOrdinal(){
		return ordinal;
	}
	
	public void setOrdinal(int ordinal){
		this.ordinal = ordinal;
	}
	public String getTitle() {
		return title;
	}
	public void setTitle(String title) {
		this.title = title;
	}
	public String getValue() {
		return value;
	}
	public void setValue(String value) {
		this.value = value;
	}
	public String getDescription() {
		return description;
	}
	public void setDescription(String description) {
		this.description = description;
	}
	public String getExtendedDescription() {
		return extendedDescription;
	}
	public void setExtendedDescription(String extendedDescription) {
		this.extendedDescription = extendedDescription;
	}
	public ViewStyle getViewStyle() {
		return viewStyle;
	}
	public void setViewStyle(ViewStyle viewStyle) {
		this.viewStyle = viewStyle;
	}
	public boolean getDefaultVisibility() {
		return defaultVisibility;
	}
	public void setDefaultVisibility(boolean defaultVisibility) {
		this.defaultVisibility = defaultVisibility;
	}
	public int getPage() {
		return page;
	}
	public void setPage(int page) {
		this.page = page;
	}		
		
	public DefaultValue getDefaultValue() {
		return defaultValue;
	}
	public void setDefaultValue(DefaultValue defaultValue) {
		this.defaultValue = defaultValue;
	}
	public String getDatatype() {
		return datatype;
	}
	public void setDatatype(String datatype) {
		this.datatype = datatype;
	}
	
	public Multiplicity getMultiplicity() {
		return multiplicity;
	}
	public void setMultiplicity(Multiplicity multiplicity) {
		this.multiplicity = multiplicity;
	}		
	
	public Object getData() {
		return data;
	}
	public void setData(Object data) {
		this.data = data;
	}
	
	public Field cloneForMultiplicity(String key, Map<String, Object> properties){
		Field newField = new Field();
		newField.id = key;
		newField.ordinal = this.ordinal;
		newField.title = this.title;
		newField.value = (String)properties.get(key);
		newField.description = this.description;
		newField.extendedDescription = this.extendedDescription;
		newField.viewStyle = this.viewStyle;
		newField.datatype = this.datatype;
		newField.defaultVisibility = this.defaultVisibility;
		newField.page = this.page;
		newField.defaultValue = this.defaultValue;
		newField.data = this.data;
		
		return newField;
	}
	
	@Override
	public entities.xmlmodels.modeldefinition.Field toDatabaseDefinition(entities.xmlmodels.modeldefinition.Field field) {
		field.setDatatype(this.datatype);
		field.setDefaultValue(this.defaultValue);
		field.setId(this.id);
		field.setMultiplicity(this.multiplicity);
		field.setOrdinal(this.ordinal);
		return field;
	}
	@Override
	public void fromDatabaseDefinition(entities.xmlmodels.modeldefinition.Field item) {
		this.datatype = item.getDatatype();
		this.defaultValue = item.getDefaultValue();
		this.multiplicity = item.getMultiplicity();
	}
	@Override
	public entities.xmlmodels.viewstyledefinition.Field toDatabaseDefinition(entities.xmlmodels.viewstyledefinition.Field field) {
		field.setDefaultVisibility(this.defaultVisibility);
		field.setDescription(this.description);
		field.setExtendedDescription(this.extendedDescription);
		field.setId(this.id);
		field.setOrdinal(this.ordinal);
		field.setTitle(this.title);
		field.setViewStyle(this.viewStyle);
		field.setData(new ModelBuilder().toFieldData(data, this.viewStyle.getRenderStyle()));
		return field;
	}
	@Override
	public void fromDatabaseDefinition(entities.xmlmodels.viewstyledefinition.Field item) {
		this.defaultVisibility = item.getDefaultVisibility();
		this.description = item.getDescription();
		this.extendedDescription = item.getExtendedDescription();
		this.id = item.getId();
		this.ordinal = item.getOrdinal();
		this.title = item.getTitle();
		this.viewStyle = item.getViewStyle();
		this.data = item.getData();
	}
	@Override
	public void fromJsonObject(Map<String, Object> properties) {
		this.value = (String)properties.get(this.id);
		this.multiplicityItems = new LinkedList<Field>();
		List<String> compositeKeys =  properties.keySet().stream().filter(keys->keys.startsWith("multiple_"+this.getId())).collect(Collectors.toList());
		for (String key : compositeKeys) {
			this.multiplicityItems.add(this.cloneForMultiplicity(key, properties));
		}
		
	}
	@Override
	public int compareTo(Object o) {
		return this.ordinal.compareTo(((Field)o).getOrdinal());
	}
	@Override
	public void fromJsonObject(Map<String, Object> properties, String path) {
		this.value = (String)properties.get(path);
	}				
	
}